// TVView.cpp : implementation of the CTVView class
//

#include "stdafx.h"
#include "TV.h"

#include "TVDoc.h"
#include "TVView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTVView

IMPLEMENT_DYNCREATE(CTVView, CTreeView)

BEGIN_MESSAGE_MAP(CTVView, CTreeView)
	//{{AFX_MSG_MAP(CTVView)
	ON_NOTIFY_REFLECT(TVN_ITEMEXPANDING, OnItemExpanding)
	ON_NOTIFY_REFLECT(TVN_BEGINDRAG, OnBegindrag)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_COMMAND(IDM_TV_NUOVO, OnNuovo)
	ON_UPDATE_COMMAND_UI(IDM_TV_NUOVO, OnUpdateNuovo)
	ON_NOTIFY_REFLECT(TVN_ENDLABELEDIT, OnEndLabelEdit)
	ON_COMMAND(IDM_TV_CANCELLA, OnCancella)
	ON_UPDATE_COMMAND_UI(IDM_TV_CANCELLA, OnUpdateCancella)
	ON_WM_CHAR()
	ON_COMMAND(IDM_TV_RINOMINA, OnRinomina)
	ON_UPDATE_COMMAND_UI(IDM_TV_RINOMINA, OnUpdateRinomina)
	ON_WM_CAPTURECHANGED()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTVView construction/destruction

CTVView::CTVView()
{
	m_fDrag = FALSE;	// Flag che identifica una operazione
						// di drag&drop in corso (TRUE)
	
	m_hTarget = NULL;	// Handle del bersaglio in una operazione
						// di drag&drop
	
	m_hTreeItem = NULL;	// Handle dell'oggetto selezionato
	m_pImageDrag = NULL;	// ImageList contenente l'icona di drag&drop
}

CTVView::~CTVView()
{
	delete m_pImageDrag;
}

BOOL CTVView::PreCreateWindow(CREATESTRUCT& cs)
{
	// Aggiungo alcuni stili alla TreeView
	cs.style |= TVS_HASLINES | TVS_LINESATROOT | TVS_HASBUTTONS | TVS_EDITLABELS;

	return CTreeView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CTVView drawing

void CTVView::OnDraw(CDC* pDC)
{
	CTVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
}

void CTVView::OnInitialUpdate()
{
	CTreeView::OnInitialUpdate();

	// Ricavo il puntatore al controllo CTreeCtrl
	// associato con CTreeView
	CTreeCtrl& TV = GetTreeCtrl();

	// Creo ed associo una ImageList alla TV
	m_ImageList.Create(IDB_IMAGELIST, 16, 1, RGB(127,127,127));
	TV.SetImageList(&m_ImageList, LVSIL_NORMAL);

	InserisciElementi(TV);

	m_hTreeItem = TV.GetSelectedItem();
}

/////////////////////////////////////////////////////////////////////////////
// CTVView diagnostics

#ifdef _DEBUG
void CTVView::AssertValid() const
{
	CTreeView::AssertValid();
}

void CTVView::Dump(CDumpContext& dc) const
{
	CTreeView::Dump(dc);
}

CTVDoc* CTVView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTVDoc)));
	return (CTVDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTVView message handlers

void CTVView::InserisciElementi(CTreeCtrl& TV)
{
	// Inserisco il root
	HTREEITEM hRoot = NULL;

	hRoot = TV.InsertItem(	TVIF_TEXT | TVIF_IMAGE |
							TVIF_PARAM | TVIF_STATE |
							TVIF_SELECTEDIMAGE,
							"Root",
							0,
							0,
							0,
							0,
							1,
							TVI_ROOT,
							TVI_LAST);

	// Se l'inserimento non ha avuto problemi
	// inserisco 5 elementi figli
	if (hRoot != NULL)
	{
		CString strElemento;

		for(int i=1;i<=5;i++)
		{
			strElemento.Format("Elemento n.%d", i);

			TV.InsertItem(	TVIF_TEXT | TVIF_IMAGE |
							TVIF_PARAM | TVIF_STATE |
							TVIF_SELECTEDIMAGE,
							strElemento, 
							2,
							2,
							0,
							0,
							i + 1,
							hRoot,
							TVI_LAST);
		}

		// Inserisco un padre
		HTREEITEM hPadre = TV.InsertItem(	TVIF_TEXT | TVIF_IMAGE |
											TVIF_PARAM | TVIF_STATE |
											TVIF_SELECTEDIMAGE,
											"Elemento Padre", 
											0,
											0,
											0,
											0,
											7,
											hRoot,
											TVI_LAST);

		// Creo altri 11 figli di questo padre
		for(i=1;i<=11;i++)
		{
			strElemento.Format("Elemento n.%d", i + 5);

			TV.InsertItem(	TVIF_TEXT | TVIF_IMAGE |
							TVIF_PARAM | TVIF_STATE |
							TVIF_SELECTEDIMAGE,
							strElemento, 
							2,
							2,
							0,
							0,
							i + 7,
							hPadre,
							TVI_LAST);
		}
	}
}

void CTVView::OnItemExpanding(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	CTreeCtrl& TV = GetTreeCtrl();

	// Se il nodo  stato espanso
	// cambio l'iconcina altrimenti
	// il nodo  stato chiuso e inserisco
	// l'iconcina iniziale
	if (pNMTreeView->action == TVE_EXPAND)
	{
		TV.SetItemImage(pNMTreeView->itemNew.hItem, 1, 1);
	}
	else
	{
		TV.SetItemImage(pNMTreeView->itemNew.hItem, 0, 0);
	}

	*pResult = 0;
}

void CTVView::OnBegindrag(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	CTreeCtrl& Tree = GetTreeCtrl();
	m_hTreeItem = pNMTreeView->itemNew.hItem;

	if (Tree.GetRootItem() == m_hTreeItem)
	{
		HCURSOR cursor = AfxGetApp()->LoadStandardCursor(IDC_NO);
		SetCursor(cursor);
		AfxMessageBox("Impossibile effettuare l'operazione di drag&drop per il Root", MB_ICONSTOP);
		return;
	}

	m_pImageDrag = Tree.CreateDragImage(m_hTreeItem);
	if (m_pImageDrag)
	{
		CRect rect;
		Tree.GetItemRect(m_hTreeItem, &rect, 1);
		CPoint ptBegin(pNMTreeView->ptDrag);
		ptBegin.x = ptBegin.x - rect.left;
		ptBegin.y = ptBegin.y - rect.top;
		m_pImageDrag->BeginDrag(0, ptBegin);
		SetCapture();
		m_pImageDrag->DragEnter(this, pNMTreeView->ptDrag);
		m_fDrag = TRUE;
	}
	else
	{
		CString strItemSel = Tree.GetItemText(m_hTreeItem);
		CString strMsg;
		strMsg = "Impossibile effettuare l'operazione di drag&drop su ";
		strMsg += strItemSel;
		AfxMessageBox(strMsg, MB_ICONSTOP);
	}

	*pResult = 0;
}

void CTVView::OnMouseMove(UINT nFlags, CPoint point) 
{
	UINT flag;
	CTreeCtrl& Tree = GetTreeCtrl();

	if (m_fDrag)
	{
		HCURSOR cursor;
		m_pImageDrag->DragMove(point);
		flag = TVHT_ONITEM;

		if ((m_hTarget = Tree.HitTest(point, &flag)) != NULL)
		{
			m_pImageDrag->DragLeave(this);
			Tree.SelectDropTarget(m_hTarget);
			m_pImageDrag->DragEnter(this, point);

			if (m_hTreeItem == m_hTarget) 
			{
				cursor = AfxGetApp()->LoadStandardCursor(IDC_NO);
				SetCursor(cursor);
			}
			else
			{
				cursor = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
				SetCursor(cursor);
			}

			AutoScroll();
		}

		RECT rectCA;
		GetClientRect(&rectCA);
		// Se il puntatore  fuori la Client Area.
		if (point.x < rectCA.left || point.x > rectCA.right || point.y < rectCA.top || point.y > rectCA.bottom)
		{
			cursor = AfxGetApp()->LoadStandardCursor(IDC_NO);
			SetCursor(cursor);
		}
	}

	CTreeView::OnMouseMove(nFlags, point);
}

void CTVView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CTreeCtrl& Tree = GetTreeCtrl();

    if (m_fDrag)
	{
		m_pImageDrag->EndDrag();
		m_pImageDrag->DragLeave(this);
		ReleaseCapture();
		DragEnd();
		m_fDrag = FALSE;
		HCURSOR cursor = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
		SetCursor(cursor);
	}

	CTreeView::OnLButtonUp(nFlags, point);
}

void CTVView::AutoScroll()
{
	if (m_fDrag)
	{
		CTreeCtrl& TV = GetTreeCtrl();
		CPoint pt;
		GetCursorPos(&pt);
		ScreenToClient(&pt);

		HTREEITEM hNext = NULL, hPrev = NULL;
		if (m_hTarget)
		{
			hPrev = TV.GetPrevVisibleItem(m_hTarget);
			hNext = TV.GetNextVisibleItem(m_hTarget);
		}

		if (hNext)
		{
			m_pImageDrag->DragLeave(this);
			UpdateWindow();
			TV.EnsureVisible(hNext);
			m_pImageDrag->DragEnter(this, pt);
		}

		if (hPrev)
		{
			m_pImageDrag->DragLeave(this);
			UpdateWindow();
			TV.EnsureVisible(hPrev);
			m_pImageDrag->DragEnter(this, pt);
		}
	}
}

void CTVView::OnNuovo() 
{
	CTreeCtrl& TV = GetTreeCtrl();
	HTREEITEM hSelect = TV.GetSelectedItem();
	int iElementi = TV.GetCount();
	CString strNome;
	strNome.Format("Elemento n.%d", iElementi - 1);

	HTREEITEM hChild = TV.InsertItem(	TVIF_TEXT  | TVIF_IMAGE |
										TVIF_PARAM | TVIF_STATE |
										TVIF_SELECTEDIMAGE,
										strNome, 
										2,
										2,
										0,
										0,
										iElementi+1,
										hSelect,
										TVI_LAST);
	if(hChild)
	{
		TV.SetItemImage(hSelect, 1, 1);
		TV.Expand(hSelect, TVE_EXPAND);
	}

}

void CTVView::OnUpdateNuovo(CCmdUI* pCmdUI) 
{
	CTreeCtrl& TV = GetTreeCtrl();
	HTREEITEM hSelect = TV.GetSelectedItem();
	if (hSelect)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CTVView::OnEndLabelEdit(NMHDR* pNMHDR, LRESULT* pResult) 
{
	TV_DISPINFO* pTVDispInfo = (TV_DISPINFO*)pNMHDR;
	CTreeCtrl& TV = GetTreeCtrl();

	TV.SetItemText(pTVDispInfo->item.hItem, pTVDispInfo->item.pszText);
	
	*pResult = 0;
}

void CTVView::DragEnd()
{
	CTreeCtrl& TV = GetTreeCtrl();

	if (m_hTarget != m_hTreeItem)
	{
		CString strNome;
		int iElementi = 0;
		strNome= TV.GetItemText(m_hTreeItem);
		iElementi = (int) TV.GetItemData(m_hTreeItem);

		TV.DeleteItem(m_hTreeItem);
		TV.SelectDropTarget((HTREEITEM)NULL);

		m_hTreeItem = TV.InsertItem(	TVIF_TEXT  | TVIF_IMAGE |
										TVIF_PARAM | TVIF_STATE |
										TVIF_SELECTEDIMAGE,
										strNome, 
										2,
										2,
										0,
										0,
										iElementi,
										m_hTarget,
										TVI_LAST);
		if(m_hTreeItem)
		{
			TV.Expand(m_hTarget, TVE_EXPAND);
			TV.SortChildren(m_hTarget);
		}
	}
}

void CTVView::OnCancella() 
{
	CTreeCtrl& TV = GetTreeCtrl();
	HTREEITEM hSelect = TV.GetSelectedItem();
	if (hSelect)
	{
		HTREEITEM hParent = TV.GetParentItem(hSelect);
		TV.DeleteItem(hSelect);
		if(hParent && !(TV.GetChildItem(hParent)))
			TV.SetItemImage(hParent, 2, 2);
	}
}

void CTVView::OnUpdateCancella(CCmdUI* pCmdUI) 
{
	CTreeCtrl& TV = GetTreeCtrl();
	HTREEITEM hSelect = TV.GetSelectedItem();
	HTREEITEM hRoot = TV.GetRootItem();

	if (hSelect == hRoot)
		pCmdUI->Enable(FALSE);
	else
	{
		if (hSelect)
			pCmdUI->Enable(TRUE);
		else
			pCmdUI->Enable(FALSE);
	}
}

void CTVView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if (nChar == VK_ESCAPE && m_fDrag)
	{
		ReleaseCapture();
		m_pImageDrag->DragLeave(this);
		m_pImageDrag->EndDrag();
		m_fDrag = FALSE;
	}
	
	CTreeView::OnChar(nChar, nRepCnt, nFlags);
}

void CTVView::OnRinomina() 
{
	CTreeCtrl& TV = GetTreeCtrl();
	HTREEITEM hSelect = TV.GetSelectedItem();
	
	if (hSelect)
		TV.EditLabel(hSelect);		
}

void CTVView::OnUpdateRinomina(CCmdUI* pCmdUI) 
{
	CTreeCtrl& TV = GetTreeCtrl();
	HTREEITEM hSelect = TV.GetSelectedItem();
	if (hSelect)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CTVView::OnCaptureChanged(CWnd *pWnd) 
{
	if (m_fDrag)
	{
		ReleaseCapture();
		m_pImageDrag->DragLeave(this);
		m_pImageDrag->EndDrag();
		m_fDrag = FALSE;
	}
	
	CTreeView::OnCaptureChanged(pWnd);
}
